"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateComponentTemplate = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const find_module_1 = require("@schematics/angular/utility/find-module");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const change_1 = require("@schematics/angular/utility/change");
function generateComponentTemplate(options) {
    return () => {
        const templateSource = (0, schematics_1.apply)((0, schematics_1.url)('./files'), [
            (0, schematics_1.applyTemplates)(Object.assign({ classify: core_1.strings.classify, dasherize: core_1.strings.dasherize }, options)),
            (0, schematics_1.move)((0, core_1.normalize)(`/${options.path}/${core_1.strings.dasherize(options.name)}`))
        ]);
        options.type = "Component";
        return (0, schematics_1.chain)([
            (0, schematics_1.externalSchematic)('@schematics/angular', 'component', {
                name: options.name,
                standalone: options.standalone,
                path: options.path,
                project: options.project,
                style: "scss",
                skipImport: !options.isPageModule
            }),
            options.isPageModule ? (host) => host : addDeclarationToModuleCpntsArr(options),
            (0, schematics_1.mergeWith)(templateSource, schematics_1.MergeStrategy.Overwrite)
        ]);
    };
}
exports.generateComponentTemplate = generateComponentTemplate;
function addDeclarationToModuleCpntsArr(options) {
    return (host) => {
        var _a;
        options.module = (0, find_module_1.findModuleFromOptions)(host, options);
        let modulePath = (_a = options.module) === null || _a === void 0 ? void 0 : _a.toString();
        let file = host.read(modulePath);
        if (!file) {
            throw new schematics_1.SchematicsException(`${modulePath} not found`);
        }
        let moduleFile = ts.createSourceFile(modulePath, file.toString(), ts.ScriptTarget.Latest, true);
        let cpntsVar = moduleFile.statements.filter(ts.isVariableStatement).find((v) => {
            return v.declarationList.declarations[0].name.getText() === "components";
        });
        let cpntsArr = (0, ast_utils_1.findNodes)(cpntsVar, ts.SyntaxKind.ArrayLiteralExpression, 1)[0];
        let occurrencesCount = cpntsArr.elements.length;
        let text = cpntsArr.getFullText(moduleFile);
        let componentVar = core_1.strings.classify(options.name) + "Component";
        let insertPos = cpntsArr.elements.pos;
        let finalVar = componentVar;
        if (occurrencesCount > 0) {
            const lastRouteLiteral = [...cpntsArr.elements].pop();
            const indentation = text.match(/\r?\n(\r?)\s*/) || [];
            const routeText = `${indentation[0] || ' '}${componentVar}`;
            insertPos = lastRouteLiteral.end;
            finalVar = `,${routeText}`;
        }
        let changes = [
            new change_1.InsertChange(modulePath, insertPos, finalVar)
        ];
        // add import path
        let filePath = addImportPath(options, modulePath, changes, moduleFile, componentVar, host);
        // add scss
        addScssToAppStyles(filePath, host);
        return host;
    };
}
function addScssToAppStyles(filePath, host) {
    let componentLocation = (0, find_module_1.buildRelativePath)("src", filePath);
    let update0 = componentLocation.split("/");
    let cpntFileName = update0.at(-1);
    let cpntName = update0.at(-2);
    let cpntNames = update0.pop().split(".");
    cpntNames.pop();
    update0.shift();
    let cssLocation = update0.join("/");
    let phoneScss = cpntFileName + ".phone.scss";
    // let tabletScss = cpntFileName.split(".ts") + ".tablet.scss";
    let globalScss = cpntFileName + ".global.scss";
    let scssFiles = [phoneScss, globalScss]
        .map((fileName) => {
        return [cssLocation, fileName].join("/");
    });
    let resultString = scssFiles.reduce((acc, x) => {
        return acc + "\n@import \"" + x + "\";";
    }, "// " + cpntName);
    let appStylesPath = "src/styles.scss";
    let appStylesFile = host.read(appStylesPath);
    if (!appStylesFile) {
        throw new schematics_1.SchematicsException(`${appStylesPath} not found`);
    }
    let appStylesFileString = appStylesFile.toString();
    let scssEndMarkerRegex = /\/\/ media-query-marker \(IMPT DONT DELETE THIS LINE\)/;
    let appStylesFileStringArray = (appStylesFileString.split(scssEndMarkerRegex));
    appStylesFileStringArray.splice(1, 0, "// media-query-marker (IMPT DONT DELETE THIS LINE)");
    appStylesFileStringArray.splice(1, 0, "\n\n");
    appStylesFileStringArray.splice(1, 0, resultString);
    let resultAppStylesFileString = appStylesFileStringArray.join("");
    host.overwrite(appStylesPath, resultAppStylesFileString);
}
function addImportPath(options, modulePath, changes, moduleFile, componentVar, host) {
    let filePath = `/${options.path}/` +
        (options.flat ? '' : core_1.strings.dasherize(options.name) + '/') +
        core_1.strings.dasherize(options.name) +
        (options.type ? '.' : '') +
        core_1.strings.dasherize(options.type);
    let importPath = (0, find_module_1.buildRelativePath)(modulePath, filePath);
    changes.push((0, ast_utils_1.insertImport)(moduleFile, modulePath, componentVar, importPath));
    let recorder = host.beginUpdate(modulePath);
    changes.forEach((addDeclaration) => {
        if (addDeclaration instanceof change_1.InsertChange) {
            recorder.insertLeft(addDeclaration.pos, addDeclaration.toAdd);
        }
    });
    host.commitUpdate(recorder);
    return filePath;
}
//# sourceMappingURL=index.js.map